import os
from subprocess import Popen, PIPE

import pandas as pd

from magnumapi.tool_adapters.ToolAdapter import ToolAdapter

print('Loaded Tool Adapter version 0.0.1 for ANSYS2020R2')


class AnsysToolAdapter(ToolAdapter):
    """ An AnsysToolAdapter class with methods to execute an ANSYS model from command line, read figures of merit table
    and prepare a Lorentz force file

    """

    def __init__(self,
                 ansys_path: str,
                 rel_dir: str,
                 inp_path: str,
                 out_path: str,
                 output_file: str
                 ) -> None:
        """ A constructor of AnsysToolAdapter instance

        :param ansys_path: a path to an ansys executable
        :param rel_dir: a relative directory with model
        :param inp_path: a path to an input file
        :param out_path: a path to an output file
        :param output_file: a path to an output file #ToDo: is it a path or a name?
        """
        self.ansys_path = ansys_path
        self.rel_dir = rel_dir
        self.inp_path = inp_path
        self.out_path = out_path
        self.output_file = output_file
        self.output_lines = []

    def run(self) -> None:
        if os.path.isfile(self.output_file):
            os.remove(self.output_file)
        else:  ## Show an error ##
            print("Warning: %s file not found" % self.output_file)
        # ToDo: -aa_r should come as a parameter
        cmd = [self.ansys_path, '-b', '-p', '-aa_r', '-smp', '-np', '2', '-lch', '-dir', self.rel_dir, '-i',
                 self.inp_path, '-o', self.out_path]
        p = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE)
        byte_output, err = p.communicate(b"input data that is passed to subprocess' stdin")

        byte_output_decode = byte_output.decode()
        self.output_lines = byte_output_decode.split('\n')

    def read_figures_of_merit_table(self) -> pd.DataFrame:
        with open(self.output_file, 'r') as output:
            output_file_content = output.read().split('\n')

        keys = output_file_content[0].split(' ')
        keys = [key for key in keys if key != '']

        values = output_file_content[1].split(' ')
        values = [float(value) for value in values if value != '']

        key_value = {key: value for key, value in zip(keys, values)}
        return pd.DataFrame(key_value, index=[0])

    @staticmethod
    def convert_figures_of_merit_to_dict(fom_df: pd.DataFrame) -> dict:
        return fom_df.to_dict('records')[0]

    @staticmethod
    def prepare_force_file(force_input_path: str, force_output_path: str) -> None:
        """ Method preparing a force file for ANSYS from a ROXIE Lorentz force file, .force2d

        :param force_input_path: an path to a ROXIE Lorentz force file
        :param force_output_path: a path to an output file for ANSYS
        """
        with open(force_input_path, 'r') as file_read:
            force_txt = file_read.readlines()

        ansys_force = []
        for force_txt_el in force_txt:
            row_float = [float(el) for el in force_txt_el.replace('\n', '').split(' ') if el != '']
            ansys_force.append('nodeNum = NODE(%f, %f, 0.0)' % tuple(row_float[:2]))
            ansys_force.append('F,nodeNum,FX, %f' % row_float[2])
            ansys_force.append('F,nodeNum,FY, %f' % row_float[3])

        with open(force_output_path, 'w') as file_write:
            for ansys_force_el in ansys_force:
                file_write.write(ansys_force_el + '\n')



import os
from abc import ABC, abstractmethod

import papermill as pm
import scrapbook as sb
from papermill import PapermillExecutionError

from magnumapi.optimization import NotebookConverter
from magnumapi.optimization.OptimizationConfig import OptimizationConfig
from magnumapi.optimization.constants import ERROR_KEY
from magnumapi.tool_adapters.DirectoryManager import DirectoryManager


class ModelExecutor(ABC):
    """An abstract class for model executor used in optimization

    """

    @staticmethod
    def initialize_folder_structure(config: OptimizationConfig) -> str:
        """ Method initializing the folder directory structure to store optimization results outside of the main
        directory.

        :return: a path to the output root directory where all results are stored.
        """
        DirectoryManager.create_directory_if_nonexistent(config.output_folder)
        subdirectory_name = DirectoryManager.find_output_subdirectory_name(config.output_folder)
        output_subdirectory_dir = os.path.join(config.output_folder, subdirectory_name)

        DirectoryManager.create_directory_if_nonexistent(output_subdirectory_dir)
        DirectoryManager.copy_model_input(config.input_folder, 'input', output_subdirectory_dir)
        DirectoryManager.copy_notebook_folders(config.input_folder, config.notebooks, output_subdirectory_dir)
        DirectoryManager.create_directory_if_nonexistent(os.path.join(output_subdirectory_dir, 'optimization'))
        DirectoryManager.copy_model_input(config.input_folder, 'optimization', output_subdirectory_dir)

        logger_path = os.path.join(output_subdirectory_dir, config.logger_rel_path)
        print('The logger is saved in: %s' % logger_path)

        for config_notebook in config.notebooks:
            input_ipynb_file_path = os.path.join(output_subdirectory_dir,
                                                 config_notebook.notebook_folder,
                                                 config_notebook.notebook_name)
            output_ipynb_file_path = os.path.join(output_subdirectory_dir,
                                                  config_notebook.notebook_folder,
                                                  config_notebook.notebook_name.lower().replace('.ipynb', '_script.py'))
            NotebookConverter.convert_notebook_to_script(input_ipynb_file_path,
                                                         config_notebook.notebook_name.lower().split('.')[0],
                                                         output_ipynb_file_path)

        return output_subdirectory_dir

    @abstractmethod
    def execute(self, model_dir: str, model_name: str, parameters_dct: dict) -> dict:
        """Method executing a model and returning figures of merit.

        :param model_dir: model directory
        :param model_name: name of a model
        :param parameters_dct: a dictionary with model execution parameters and corresponding values

        :return: a dictionary with figures of merit if the computation was successful, otherwise an empty dictionary
        """
        raise NotImplementedError('This method is not implemented for this class')


class ScriptModelExecutor(ModelExecutor):
    """ An implementation of ModelExecutor abstract class for scripts

    """

    def execute(self, model_dir: str, model_name: str, parameters_dct: dict) -> dict:
        """Method calculating figures of merit with scripts. Notebooks are converted to scripts.

        :param model_dir: model directory
        :param model_name: name of a model
        :param parameters_dct: a dictionary with model execution parameters and corresponding values

        :return: a dictionary with figures of merit if the computation was successful, otherwise a dictionary with -1
        error code is returned.
        """
        script = model_name.split('.')[0].lower() + '_script'
        cwd = os.getcwd()

        os.chdir(model_dir)
        run = getattr(__import__(script), 'run_' + script)
        print('Running %s script' % script)
        try:
            fom_model = run(**parameters_dct)
        except Exception as exception:
            print(exception)
            return {ERROR_KEY: -1}
        os.chdir(cwd)

        return fom_model


class NotebookModelExecutor(ModelExecutor):
    """ An implementation of ModelExecutor abstract class for notebooks

    """
    def execute(self, model_dir: str, model_name: str, parameters_dct: dict) -> dict:
        """Method calculating figures of merit with notebooks (papermill and scrapbook packages).

        :return: a dictionary with figures of merit if the computation was successful, otherwise a dictionary with -1
        error code is returned.
        """
        notebook_path = os.path.join(model_dir, model_name)
        notebook_name_split = model_name.split('.')
        out_notebook_name = '%s_out.%s' % tuple(notebook_name_split)

        out_notebook_path = os.path.join(model_dir, out_notebook_name)

        try:
            pm.execute_notebook(notebook_path, out_notebook_path, cwd=model_dir, parameters=parameters_dct)
        except PapermillExecutionError as e:
            # on error print the message
            print(e.exec_count)
            print(e.source)
            print(e.traceback[-1])
            return {ERROR_KEY: -1}
        except Exception as exception:
            raise Exception(exception)

        # fetch figure of merit
        return sb.read_notebook(out_notebook_path).scraps['model_results'].data


class ModelExecutorFactory:
    """ A factory class returning either a script or notebook executor

    """
    @staticmethod
    def build(is_script_executed: bool) -> "ModelExecutor":
        """

        :param is_script_executed: True if notebooks are executed as script, False otherwise
        :return: ScriptModelExecutor instance if True, otherwise NotebookModelExecutor
        """
        if is_script_executed:
            return ScriptModelExecutor()
        else:
            return NotebookModelExecutor()



from enum import Enum, auto
from typing import List, Dict, Type
import json

import pandas as pd

from magnumapi.geometry.SlottedGeometry import SlottedGeometry, SlottedRelativeCosThetaGeometry
from magnumapi.geometry.blocks.CosThetaBlock import AbsoluteCosThetaBlock, RelativeCosThetaBlock
from magnumapi.geometry.Geometry import Geometry, RelativeCosThetaGeometry
from magnumapi.geometry.blocks.RectangularBlock import RectangularBlock
from magnumapi.geometry.definitions.AbsoluteCosThetaBlockDefinition import AbsoluteCosThetaBlockDefinition
from magnumapi.geometry.definitions.LayerDefinition import LayerDefinition, SlottedLayerDefinition
from magnumapi.geometry.definitions.RectangularBlockDefinition import RectangularBlockDefinition
from magnumapi.geometry.definitions.RelativeCosThetaBlockDefinition import RelativeCosThetaBlockDefinition
from magnumapi.cadata.CableDatabase import CableDatabase
import magnumapi.tool_adapters.roxie.RoxieAPI as RoxieAPI


# ToDo: BuilderDesign pattern:
# - with_geometry_type - use single dispatch mechanism (dict or dataframe as input; ABSOLUTE, RELATIVE as output) - this actually is deduced when block def is provided
# - with_aperture_radius - either a value or none
# - with_block_definition - use a single dispatch mechanism (dict or dataframe or list of definitions as input; List of Block definitions as output) - from that one can deduce what is the geometry type - get_keys() also as a single dispatch
# - with_layer_definition - use a single dispatch mechanism (dict or dataframe or list of defitnitions as input; List of Layer definitions as output) - from that one can deduce what is the layer type
# use generators to iterate over a dict and a dataframe to use the same for loop!!! - also with a single dispatch mechanism for the generator
# param = {'block_defs': self.block_defs, 'layer_defs': self.layer_defs}
# if self.r_aperture:
#     param['r_aperture'] = r_aperture
# return object_dct[geometry_type](**param)
# geometry_type is 'Absolute, Relative, SlottedAbsolute, SlottedRelative'

class GeometryType(Enum):
    """GeometryType is an enum type used to distinguish between an absolute and relative geometry.

    """
    ABSOLUTE = auto()
    RELATIVE = auto()


class GeometryFactory:
    """ GeometryFactory implements a factory design pattern and is used to produce:
    - rectangular geometry
    - absolute cos-theta geometry
    - relative cos-theta geometry

    """

    # A dictionary from geometry type to block type
    geometry_type_to_block = {1: AbsoluteCosThetaBlock,
                              2: RectangularBlock}

    # A dictionary from geometry type to block definition type
    geometry_type_to_block_definition = {1: AbsoluteCosThetaBlockDefinition,
                                         2: RectangularBlockDefinition}

    @classmethod
    def init_with_json(cls, json_file_path: str, cadata: CableDatabase) -> Type[Geometry]:
        """ Class method initializing a Geometry instance from a JSON file.

        :param json_file_path: a path to a json file
        :param cadata: a CableDatabase instance
        :return: initialized geometry instance
        """
        json_content = cls.read_json_file(json_file_path)
        block_dct_defs = json_content['block_defs']
        layer_dct_defs = json_content['layer_defs']
        r_aperture = json_content.get('r_aperture', None)
        return cls.init_with_dict(block_dct_defs, layer_dct_defs, cadata, r_aperture=r_aperture)

    @staticmethod
    def read_json_file(json_file_path: str) -> dict:
        """ Static method reading a json file and returning a list of dictionaries with block definitions.

        :param json_file_path: a path to a json file
        :return: a list of dictionaries with geometry definition (block definition)
        """
        with open(json_file_path) as f:
            return json.load(f)

    @classmethod
    def init_with_dict(cls,
                       block_dct_defs: List[Dict],
                       layer_dct_defs: List[Dict],
                       cadata: CableDatabase,
                       r_aperture=None) -> Type[Geometry]:
        """ Class method initializing a Geometry instance from a list of dictionaries with block definition.

        :param block_dct_defs: a list of dictionaries with geometry definition (block definition)
        :param layer_dct_defs: a list of dictionaries with layer definitions
        :param cadata: a CableDatabase instance
        :param r_aperture: aperture radius in mm, None by default
        :return: initialized geometry instance
        """
        geom_type = cls.retrieve_geometry_type_dict(block_dct_defs)

        if geom_type == GeometryType.ABSOLUTE:
            return cls.init_absolute_with_dict(block_dct_defs, layer_dct_defs, cadata, r_aperture=r_aperture)

        if geom_type == GeometryType.RELATIVE:
            return cls.init_relative_with_dict(block_dct_defs, layer_dct_defs, cadata, r_aperture=r_aperture)

    @staticmethod
    def retrieve_geometry_type_dict(block_dct_defs: List[Dict]) -> GeometryType:
        """ Static method returning a geometry type enumeration for a given list of block definitions.
        An AttributeError is raised in case of inconsistencies in the definition.

        :param block_dct_defs: list of block definitions
        :return: a geometry type enumeration
        """
        if all(['alpha_r' in dct.keys() for dct in block_dct_defs]):
            return GeometryType.RELATIVE
        elif any(['alpha_r' in dct.keys() for dct in block_dct_defs]):
            raise AttributeError(
                'Error, inconsistent geometry definition. '
                'The geometry definition should consist of either all relative definitions or none.')
        else:
            return GeometryType.ABSOLUTE

    @classmethod
    def init_absolute_with_dict(cls,
                                block_defs: List[Dict],
                                layer_dct_defs: List[Dict],
                                cadata: CableDatabase,
                                r_aperture=None) -> Type[Geometry]:
        """ Static method initializing an absolute geometry (either rectangular or cos-theta) from a list of block
        definitions.

        :param block_defs: a list of block definitions
        :param layer_dct_defs: a list of dictionaries with layer definitions
        :param cadata: a CableDatabase instance
        :param r_aperture: aperture radius in mm, None by default
        :return: initialized absolute geometry instance
        """
        blocks = []
        for block_def in block_defs:
            BlockClass = GeometryFactory.geometry_type_to_block[block_def['type']]
            BlockDefinitionClass = GeometryFactory.geometry_type_to_block_definition[block_def['type']]
            block_abs = BlockDefinitionClass(**block_def)
            block = BlockClass(block_def=block_abs,
                               cable_def=cadata.get_cable_definition(block_abs.condname),
                               insul_def=cadata.get_insul_definition(block_abs.condname),
                               strand_def=cadata.get_strand_definition(block_abs.condname),
                               conductor_def=cadata.get_conductor_definition(block_abs.condname))

            blocks.append(block)

        if r_aperture:
            layer_defs = cls.init_slotted_layer_defs_with_dict(layer_dct_defs)
            return SlottedGeometry(r_aperture=r_aperture, blocks=blocks, layer_defs=layer_defs)
        else:
            layer_defs = cls.init_layer_defs_with_dict(layer_dct_defs)
            return Geometry(blocks=blocks, layer_defs=layer_defs)

    @staticmethod
    def init_slotted_layer_defs_with_dict(layer_dct_defs: List[Dict]) -> List[SlottedLayerDefinition]:
        """ Static method initializing a list of slotted layer definitions from a dataframe.

        :param layer_dct_defs: a list with layer definitions
        :return: a list of layer definitions
        """
        return [SlottedLayerDefinition(**layer_dct_def) for layer_dct_def in layer_dct_defs]

    @staticmethod
    def init_layer_defs_with_dict(layer_dct_defs: List[Dict]) -> List[LayerDefinition]:
        """ Static method initializing a list of layer definitions from a dataframe.

        :param layer_dct_defs: a list with layer definitions
        :return: a list of layer definitions
        """
        return [LayerDefinition(**layer_dct_def) for layer_dct_def in layer_dct_defs]

    @classmethod
    def init_relative_with_dict(cls,
                                block_dct_defs: List[Dict],
                                layer_dct_defs: List[Dict],
                                cadata: CableDatabase,
                                r_aperture=None) -> Geometry:
        """ Static method initializing an absolute cos-theta geometry from a list of block definitions.

        :param block_dct_defs: a list of block definitions
        :param layer_dct_defs: a list of dictionaries with layer definitions
        :param cadata: a CableDatabase instance
        :param r_aperture: aperture radius in mm, None by default
        :return: initialized relative geometry instance
        """
        blocks = []
        for block_def in block_dct_defs:
            block_rel = RelativeCosThetaBlockDefinition(**block_def)
            block = RelativeCosThetaBlock(block_def=block_rel,
                                          cable_def=cadata.get_cable_definition(block_rel.condname),
                                          insul_def=cadata.get_insul_definition(block_rel.condname),
                                          strand_def=cadata.get_strand_definition(block_rel.condname),
                                          conductor_def=cadata.get_conductor_definition(block_rel.condname))
            blocks.append(block)

        if r_aperture:
            layer_defs = cls.init_slotted_layer_defs_with_dict(layer_dct_defs)
            return SlottedRelativeCosThetaGeometry(r_aperture=r_aperture, blocks=blocks, layer_defs=layer_defs)
        else:
            layer_defs = cls.init_layer_defs_with_dict(layer_dct_defs)
            return RelativeCosThetaGeometry(blocks=blocks, layer_defs=layer_defs)

    @classmethod
    def init_with_data(cls, data_file_path: str, cadata: CableDatabase, r_aperture=None) -> Geometry:
        """ Class method initializing a Geometry instance from a DATA ROXIE file.

        :param data_file_path: a path to a json file
        :param cadata: a CableDatabase instance
        :param r_aperture: aperture radius in mm, None by default
        :return: initialized geometry instance
        """
        block_df = RoxieAPI.read_bottom_header_table(data_file_path, keyword='BLOCK')
        layer_df = RoxieAPI.read_nested_bottom_header_table(data_file_path, keyword='LAYER')
        return cls.init_with_df(block_df, layer_df, cadata, r_aperture=r_aperture)

    @classmethod
    def init_with_csv(cls,
                      block_csv_file_path: str,
                      layer_csv_file_path: str,
                      cadata: CableDatabase,
                      r_aperture=None) -> Geometry:
        """ Class method initializing a Geometry instance from a CSV file.

        :param block_csv_file_path: a path to a csv file with block definitions
        :param layer_csv_file_path: a path to a csv file with layer definitions
        :param cadata: a CableDatabase instance
        :return: initialized geometry instance
        """
        block_df = pd.read_csv(block_csv_file_path, index_col=0)
        layer_df = pd.read_csv(layer_csv_file_path, index_col=0)
        return cls.init_with_df(block_df, layer_df, cadata, r_aperture=r_aperture)

    @classmethod
    def init_with_df(cls, block_df: pd.DataFrame, layer_df, cadata: CableDatabase, r_aperture=None) -> Geometry:
        """ Class method initializing a Geometry instance from a dataframe with block definition.

        :param block_df: a dataframe with geometry definition (block definition)
        :param cadata: a CableDatabase instance
        :param r_aperture: aperture radius in mm, None by default
        :return: initialized geometry instance
        """
        geom_type = cls.retrieve_geometry_type_df(block_df)
        if geom_type == GeometryType.ABSOLUTE:
            return cls.init_absolute_with_df(block_df, layer_df, cadata, r_aperture=r_aperture)

        if geom_type == GeometryType.RELATIVE:
            return cls.init_relative_with_df(block_df, layer_df, cadata, r_aperture=r_aperture)

    @staticmethod
    def retrieve_geometry_type_df(block_df: pd.DataFrame) -> "GeometryType":
        """ Static method returning a geometry type enumeration for a given dataframe with block definitions.
        An AttributeError is raised in case of inconsistencies in the definition.

        :param block_defs: list of block definitions
        :return: a geometry type enumeration
        """
        if ('alpha_r' in block_df.columns) and ('phi_r' in block_df.columns):
            return GeometryType.RELATIVE
        elif ('alpha_r' in block_df.columns) or ('phi_r' in block_df.columns):
            raise AttributeError('Error, inconsistent geometry definition')
        else:
            return GeometryType.ABSOLUTE

    @classmethod
    def init_absolute_with_df(cls,
                              block_df: pd.DataFrame,
                              layer_df: pd.DataFrame,
                              cadata: CableDatabase,
                              r_aperture=None) -> Geometry:
        """ Static method initializing an absolute geometry (either rectangular or cos-theta) from a dataframe with
        block definitions.

        :param block_df: a dataframe with block definitions
        :param layer_defs: a dataframe with layer definitions
        :param cadata: a CableDatabase instance
        :param r_aperture: aperture radius in mm, None by default
        :return: initialized absolute geometry instance
        """
        blocks = []
        for _, row in block_df.iterrows():
            BlockClass = GeometryFactory.geometry_type_to_block[row['type']]
            BlockDefinitionClass = GeometryFactory.geometry_type_to_block_definition[row['type']]
            if row['type'] == 2:
                row = row.rename(BlockClass.roxie_to_magnum_dct)

            block_def = BlockDefinitionClass(**row.to_dict())
            block = BlockClass(block_def=block_def,
                               cable_def=cadata.get_cable_definition(block_def.condname),
                               insul_def=cadata.get_insul_definition(block_def.condname),
                               strand_def=cadata.get_strand_definition(block_def.condname),
                               conductor_def=cadata.get_conductor_definition(block_def.condname))

            blocks.append(block)

        if r_aperture:
            layer_defs = cls.init_slotted_layer_defs_with_df(layer_df)
            return SlottedGeometry(blocks=blocks, layer_defs=layer_defs, r_aperture=r_aperture)
        else:
            layer_defs = cls.init_layer_defs_with_df(layer_df)
            return Geometry(blocks=blocks, layer_defs=layer_defs)

    @staticmethod
    def init_slotted_layer_defs_with_df(layer_df: pd.DataFrame) -> List[SlottedLayerDefinition]:
        """ Static method initializing a list of slotted layer definitions from a dataframe.

        :param layer_df: a dataframe with layer definitions
        :return: a list of layer definitions
        """
        return [SlottedLayerDefinition(**row.to_dict()) for _, row in layer_df.iterrows()]

    @staticmethod
    def init_layer_defs_with_df(layer_df: pd.DataFrame) -> List[LayerDefinition]:
        """ Static method initializing a list of layer definitions from a dataframe.

        :param layer_df: a dataframe with layer definitions
        :return: a list of layer definitions
        """
        return [LayerDefinition(**row.to_dict()) for _, row in layer_df.iterrows()]

    @classmethod
    def init_relative_with_df(cls,
                              block_df: pd.DataFrame,
                              layer_df: pd.DataFrame,
                              cadata: CableDatabase,
                              r_aperture=None) -> Geometry:
        """ Static method initializing an absolute cos-theta geometry from a dataframe with block definitions.

        :param block_df: a dataframe with block definitions
        :param layer_df: a dataframe with layer definitions
        :param cadata: a CableDatabase instance
        :param r_aperture: aperture radius in mm, None by default
        :return: initialized relative geometry instance
        """
        blocks = []
        for row_dict in block_df.to_dict('records'):
            block_rel = RelativeCosThetaBlockDefinition(**row_dict)

            block = RelativeCosThetaBlock(block_def=block_rel,
                                          cable_def=cadata.get_cable_definition(block_rel.condname),
                                          insul_def=cadata.get_insul_definition(block_rel.condname),
                                          strand_def=cadata.get_strand_definition(block_rel.condname),
                                          conductor_def=cadata.get_conductor_definition(block_rel.condname))

            blocks.append(block)

        if r_aperture:
            layer_defs = cls.init_slotted_layer_defs_with_df(layer_df)
            return SlottedRelativeCosThetaGeometry(blocks=blocks, layer_defs=layer_defs, r_aperture=r_aperture)
        else:
            layer_defs = cls.init_layer_defs_with_df(layer_df)
            return RelativeCosThetaGeometry(blocks=blocks, layer_defs=layer_defs)



import unittest

from magnumapi.tool_adapters.ansys.AnsysToolAdapter import AnsysToolAdapter
from magnumapi.tool_adapters.roxie.RoxieToolAdapter import RoxieToolAdapter
from tests.resource_files import create_resources_file_path


class TestAnsysToolAdapter(unittest.TestCase):

    def test_roxie_force_to_ansys(self):
        # arrange
        roxie_force_input_path = create_resources_file_path('resources/tool_adapters/ansys/roxie.force2d')
        roxie_force_output_path = create_resources_file_path('resources/tool_adapters/ansys/roxie_edit.force2d')
        ansys_force_output_path = create_resources_file_path('resources/tool_adapters/ansys/forces_edit.vallone')
        field = 16
        target_field = 15

        # act
        # # update ROXIE force file
        RoxieToolAdapter.update_force2d_with_field_scaling(roxie_force_input_path,
                                                           roxie_force_output_path,
                                                           field,
                                                           target_field)

        # # prepare ANSYS force file
        AnsysToolAdapter.prepare_force_file(roxie_force_output_path, ansys_force_output_path)

        # assert
        with open(ansys_force_output_path, 'r') as file:
            ansys_force = file.readlines()

        output_text_ref_path = create_resources_file_path('resources/tool_adapters/ansys/forces_edit_ref.vallone')

        with open(output_text_ref_path, 'r') as file:
            ansys_force_ref = file.readlines()

        for ansys_force_ref_el, ansys_force_el in zip(ansys_force_ref, ansys_force):
            self.assertEqual(ansys_force_ref_el, ansys_force_el)


if __name__ == '__main__':
    unittest.main()
